@@ -105,6 +105,7 @@
 #  define SCOPVAL_increment(ref,val)       (mpz_add_ui((ref),(val),1))
 #  define SCOPVAL_addto(ref,val1,val2)     (mpz_add((ref),(val1),(val2)))
 #  define SCOPVAL_multo(ref,val1,val2)     (mpz_mul((ref),(val1),(val2)))
+#  define SCOPVAL_divto(ref,val1,val2)     (mpz_div((ref),(val1),(val2)))
 #  define SCOPVAL_add_int(ref,val,vint)    (mpz_add_ui((ref),(val),(long)(vint)))
 #  define SCOPVAL_subtract(ref,val1,val2)  (mpz_sub((ref),(val1),(val2)))
 #  define SCOPVAL_oppose(ref,val)          (mpz_neg((ref),(val)))
@@ -141,6 +142,7 @@
 #  define SCOPVAL_increment(ref,val)       ((ref) = (val)+(scoplib_int_t)(1))
 #  define SCOPVAL_addto(ref,val1,val2)     ((ref) = (val1)+(val2))
 #  define SCOPVAL_multo(ref,val1,val2)     ((ref) = (val1)*(val2))
+#  define SCOPVAL_divto(ref,val1,val2)     ((ref) = (val1)/(val2))
 #  define SCOPVAL_add_int(ref,val,vint)    ((ref) = (val)+(scoplib_int_t)(vint))
 #  define SCOPVAL_subtract(ref,val1,val2)  ((ref) = (val1)-(val2))
 #  define SCOPVAL_oppose(ref,val)          ((ref) = (-(val)))
