@@ -466,20 +466,25 @@ scoplib_scop_tag_content_from_string(char* str, char* tag, char* endtag)
   int lentag;
   char* res = NULL;
 
-  if (str)
+  if (str && *str && tag && *tag && endtag && *endtag)
     {
       start = str;
       lentag = strlen(tag);
-      for (; start && *start && strncmp(start, tag, lentag); ++start)
-	;
+      for (; start && *start; ++start)
+	if (*start == *tag && strlen(start) >= lentag)
+	  if (! strncmp(start, tag, lentag))
+	    break;
+
       /* The tag 'tag' was not found.*/
       if (! *start)
 	return NULL;
       start += lentag;
       stop = start;
       lentag = strlen(endtag);
-      for (size = 0; *stop && strncmp(stop, endtag, lentag); ++stop, ++size)
-	;
+      for (size = 0; stop && *stop; ++stop, ++size)
+	if (*stop == *endtag && strlen(stop) >= lentag)
+	  if (! strncmp(stop, endtag, lentag))
+	    break;
       /* the tag 'endtag' was not found. */
       if (! *stop)
 	return NULL;
@@ -583,7 +588,7 @@ scoplib_scop_read(FILE* file)
 	  tmpbuff[0] == '#' || isspace(tmpbuff[0]) || tmpbuff[0] != '<'))
     ;
   /* Store the remainder of the file, if any. */
-  if (tmpbuff[0])
+  if (! feof(file) && tmpbuff[0])
     {
       int count = strlen(tmpbuff);
       int pos = 0;
@@ -599,7 +604,6 @@ scoplib_scop_read(FILE* file)
       while ((count = fread(tmpbuff, sizeof(char), SCOPLIB_MAX_STRING, file))
 	     > 0);
     }
-
   /* Count the number of referenced arrays/variables. */
   scop->nb_arrays = 0;
   for (stmt = scop->statement; stmt; stmt = stmt->next)
@@ -760,11 +764,40 @@ scoplib_scop_free(scoplib_scop_p scop)
 
 
 
+/**
+ * scoplib_scop_shallow_free function:
+ *
+ * This function frees the allocated memory for a scoplib_scop_t
+ * structure, without touching the content of arrays, parameters and
+ * iterators fields.
+ *
+ * \param scop The pointer to the scop we want to free.
+ **
+ * - 06/07/2011: first version.
+ */
+void
+scoplib_scop_shallow_free(scoplib_scop_p scop)
+{
+  if (scop != NULL)
+  {
+    scoplib_matrix_free(scop->context);
+    if (scop->parameters != NULL)
+      free(scop->parameters);
+    if (scop->arrays != NULL)
+      free(scop->arrays);
+    scoplib_statement_shallow_free(scop->statement);
+    free(scop->optiontags);
+    free(scop);
+  }
+}
+
+
+
 /**
  * scoplib_scop_dup function:
  * This function returns a fresh identical (non shadow) copy of the
  * input scop.
- * \param scop   The scop whose information have to be printed.
+ * \param scop   The scop whose information have to be duplicated.
  **
  */
 scoplib_scop_p
@@ -776,13 +809,15 @@ scoplib_scop_dup(scoplib_scop_p scop)
   scoplib_scop_p ret = scoplib_scop_malloc();
   ret->context = scoplib_matrix_copy(scop->context);
   ret->nb_parameters = scop->nb_parameters;
-  ret->parameters = (char**) malloc(sizeof(char*) * ret->nb_parameters);
+  ret->parameters = (char**) malloc(sizeof(char*) * (ret->nb_parameters + 1));
   for (i = 0; i < ret->nb_parameters; ++i)
     ret->parameters[i] = strdup(scop->parameters[i]);
+  ret->parameters[i] = NULL;
   ret->nb_arrays = scop->nb_arrays;
-  ret->arrays = (char**) malloc(sizeof(char*) * ret->nb_arrays);
+  ret->arrays = (char**) malloc(sizeof(char*) * (ret->nb_arrays + 1));
   for (i = 0; i < ret->nb_arrays; ++i)
     ret->arrays[i] = strdup(scop->arrays[i]);
+  ret->arrays[i] = NULL;
 
   for (stm = scop->statement; stm; stm = stm->next)
     {
@@ -793,9 +828,11 @@ scoplib_scop_dup(scoplib_scop_p scop)
       newstm->read = scoplib_matrix_copy(stm->read);
       newstm->write = scoplib_matrix_copy(stm->write);
       newstm->nb_iterators = stm->nb_iterators;
-      newstm->iterators = (char**) malloc(sizeof(char*) * newstm->nb_iterators);
+      newstm->iterators = (char**) malloc(sizeof(char*) *
+					  (newstm->nb_iterators + 1));
       for (i = 0; i < newstm->nb_iterators; ++i)
 	newstm->iterators[i] = strdup(stm->iterators[i]);
+      newstm->iterators[i] = NULL;
       newstm->body = strdup (stm->body);
       if (ret->statement == NULL)
 	ret->statement = tmp = newstm;
@@ -811,3 +848,186 @@ scoplib_scop_dup(scoplib_scop_p scop)
 
   return ret;
 }
+
+/**
+ * scoplib_scop_shallow_copy function:
+ *
+ * This function returns a mixed deep and shallow copy of the
+ * input scop. Items in arrays, parameters, stm->iterators and stm->body are not
+ * duplicated.
+ *
+ * \param scop   The scop whose information have to be duplicated.
+ **
+ */
+scoplib_scop_p
+scoplib_scop_shallow_copy(scoplib_scop_p scop)
+{
+  int i;
+  scoplib_statement_p stm;
+  scoplib_statement_p tmp = NULL;
+  scoplib_scop_p ret = scoplib_scop_malloc();
+  ret->context = scoplib_matrix_copy(scop->context);
+  ret->nb_parameters = scop->nb_parameters;
+  ret->parameters = (char**) malloc(sizeof(char*) * (ret->nb_parameters + 1));
+  for (i = 0; i < ret->nb_parameters; ++i)
+    ret->parameters[i] = scop->parameters[i];
+  ret->parameters[i] = NULL;
+  ret->nb_arrays = scop->nb_arrays;
+  ret->arrays = (char**) malloc(sizeof(char*) * (ret->nb_arrays + 1));
+  for (i = 0; i < ret->nb_arrays; ++i)
+    ret->arrays[i] = scop->arrays[i];
+  ret->arrays[i] = NULL;
+
+  for (stm = scop->statement; stm; stm = stm->next)
+    {
+      scoplib_statement_p newstm = scoplib_statement_malloc();
+      newstm->domain = scoplib_matrix_list_malloc();
+      newstm->domain->elt = scoplib_matrix_copy(stm->domain->elt);
+      newstm->schedule = scoplib_matrix_copy(stm->schedule);
+      newstm->read = scoplib_matrix_copy(stm->read);
+      newstm->write = scoplib_matrix_copy(stm->write);
+      newstm->nb_iterators = stm->nb_iterators;
+      newstm->iterators = (char**) malloc(sizeof(char*) *
+					  (newstm->nb_iterators + 1));
+      for (i = 0; i < newstm->nb_iterators; ++i)
+	newstm->iterators[i] = stm->iterators[i];
+      newstm->iterators[i] = NULL;
+      newstm->body = stm->body;
+      if (ret->statement == NULL)
+	ret->statement = tmp = newstm;
+      else
+	{
+	  tmp->next = newstm;
+	  tmp = tmp->next;
+	}
+    }
+  if (scop->optiontags)
+    ret->optiontags = strdup(scop->optiontags);
+  ret->usr = scop->usr;
+
+  return ret;
+}
+
+
+
+
+/**
+ * scoplib_scop_normalize_schedule function:
+ *
+ * This function normalize the beta part of the 2d+1 schedules
+ *
+ * \param scop   The scop whose information have to be printed.
+ **
+ */
+void
+scoplib_scop_normalize_schedule(scoplib_scop_p scop)
+{
+  if (scop == NULL)
+    return;
+  // 1- Ensure the schedules are all 2d+1.
+  scoplib_statement_p stm;
+  int max_dim = 0;
+  int nb_stmts = 0;
+  int i, j, k, l;
+  for (stm = scop->statement; stm; stm = stm->next)
+    {
+      if (stm->schedule->NbRows != 2 * stm->nb_iterators + 1)
+	{
+	  fprintf (stderr, "[ScopLib] Schedules are not 2d+1\n");
+	  return;
+	  for (i = 0; i < stm->nb_iterators; ++i)
+	    if (SCOPVAL_get_si(stm->schedule->p[2 * i + 1][i + 1]) != 1)
+	      {
+		fprintf (stderr, "[ScopLib] Schedules are not 2d+1\n");
+		return;
+	      }
+	}
+      max_dim = max_dim < stm->nb_iterators ? stm->nb_iterators : max_dim;
+      ++nb_stmts;
+    }
+  ++max_dim;
+
+  // 2- Collect original beta vectors.
+  int beta[max_dim][nb_stmts];
+  for (i = 0; i < max_dim; ++i)
+    {
+      int pos = 0;
+      for (stm = scop->statement; stm; stm = stm->next)
+	{
+	  if (i <= stm->nb_iterators)
+	    beta[i][pos++] =
+	      stm->schedule->p[2 * i][stm->schedule->NbColumns - 1];
+	  else
+	    beta[i][pos++] = 0;
+	}
+    }
+
+  // 3- Lex-sort.
+  scoplib_statement_p stmts_sort[nb_stmts];
+  int pos;
+  for (stm = scop->statement, pos = 0; stm; stm = stm->next)
+    stmts_sort[pos++] = stm;
+
+  for (i = 0; i < max_dim; ++i)
+    for (j = 0; j < nb_stmts - 1; ++j)
+      for (k = j + 1; k < nb_stmts; ++k)
+	{
+	  for (l = 0; l <= i; ++l)
+	    if (beta[l][j] != beta[l][k])
+	      break;
+	  if (l > i && beta[i][j] > beta[i][k])
+	    {
+	      scoplib_statement_p tmps = stmts_sort[j];
+	      int tmp = beta[i][j];
+	      beta[i][j] = beta[i][k];
+	      stmts_sort[j] = stmts_sort[k];
+	      beta[i][k] = tmp;
+	      stmts_sort[k] = tmps;
+	    }
+	}
+
+  // 4- Normalize.
+  int last_beta[max_dim];
+  int cur_norm[max_dim];
+  for (i = 0; i < max_dim; ++i)
+    cur_norm[i] = 0;
+
+  for (i = 0; i < max_dim; ++i)
+    {
+      last_beta[i] = beta[i][0];
+      beta[i][0] = cur_norm[i];
+    }
+  for (i = 1; i < nb_stmts; ++i)
+    {
+      for (j = 0; j < max_dim; ++j)
+	if (beta[j][i] != last_beta[j])
+	  break;
+      for (k = 0; k < j; ++k)
+	{
+	  last_beta[k] = beta[k][i];
+	  beta[k][i] = cur_norm[k];
+	}
+      last_beta[j] = beta[j][i];
+      beta[j][i] = ++(cur_norm[j]);
+      for (++k; k < max_dim; ++k)
+	{
+	  last_beta[k] = beta[k][i];
+	  cur_norm[k] = 0;
+	  beta[k][i] = cur_norm[k];
+	}
+    }
+
+  // 5- Update the value in the original schedule.
+  for (i = 0; i < max_dim; ++i)
+    for (stm = scop->statement; stm; stm = stm->next)
+      {
+	if (2 * i < stm->schedule->NbRows)
+	  {
+	    // Find its beta value.
+	    for (j = 0; stmts_sort[j] != stm; ++j)
+	      ;
+	    SCOPVAL_set_si(stm->schedule->p[2 * i][stm->schedule->NbColumns -1],
+			   beta[i][j]);
+	  }
+      }
+}
